@@ -38,32 +38,13 @@ namespace Battle
 	return Rand::Get(3, 6) + (11 * Rand::Get(1, 7));
     }
 
-    u16 WideCost(u8 where, u8 whereto)
+    bool WideDifficultDirection(u8 where, u8 whereto)
     {
 	return
 	((TOP_LEFT == where) && (whereto & (LEFT | TOP_RIGHT))) ||
 	((TOP_RIGHT == where) && (whereto & (RIGHT | TOP_LEFT))) ||
 	((BOTTOM_LEFT == where) && (whereto & (LEFT | BOTTOM_RIGHT))) ||
-	((BOTTOM_RIGHT == where) && (whereto & (RIGHT | BOTTOM_LEFT))) ? 150 : 100;
-    }
-
-
-    bool CheckExtConditionPassable(const Unit & b, s16 center, s16 from)
-    {
-	const Castle* castle = Arena::GetCastle();
-	const Bridge* bridge = Arena::GetBridge();
-
-	// skip moat position
-	if(castle && castle->isBuild(BUILD_MOAT) &&
-	    Board::isMoatIndex(from) && b.GetHeadIndex() != from)
-	    return false;
-
-	// skip bridge position
-	if(bridge && ! bridge->isPassable(b.GetColor()) &&
-	    Board::isBridgeIndex(center))
-	    return false;
-
-	return true;
+	((BOTTOM_RIGHT == where) && (whereto & (RIGHT | BOTTOM_LEFT)));
     }
 }
 
@@ -195,6 +176,8 @@ struct bcell_t
 
 Battle::Indexes Battle::Board::GetAStarPath(const Unit & b, const Position & dst, bool debug)
 {
+    const Castle* castle = Arena::GetCastle();
+    const Bridge* bridge = Arena::GetBridge();
     std::map<s16, bcell_t> list;
     s16 cur = b.GetHeadIndex();
 
@@ -202,6 +185,7 @@ Battle::Indexes Battle::Board::GetAStarPath(const Unit & b, const Position & dst
     list[cur].cost = 0;
     list[cur].open = false;
 
+
     while(cur != dst.GetHead()->GetIndex())
     {
 	const Cell & center = at(cur);
@@ -215,10 +199,12 @@ Battle::Indexes Battle::Board::GetAStarPath(const Unit & b, const Position & dst
     	    Cell & cell = at(*it);
 
 	    if(list[*it].open && cell.isPassable4(b, center) &&
-		CheckExtConditionPassable(b, *it, center.GetIndex()))
+		// check bridge
+	        (!bridge || !Board::isBridgeIndex(*it) || bridge->isPassable(b.GetColor())))
 	    {
-		const s16 cost = b.isWide() ?
-		    WideCost(center.GetDirection(), GetDirection(*it, cur)) : 100;
+		const s16 cost = 100 * Board::GetDistance(*it, dst.GetHead()->GetIndex()) +
+		    (b.isWide() && WideDifficultDirection(center.GetDirection(), GetDirection(*it, cur)) ? 100 : 0) +
+		    (castle && castle->isBuild(BUILD_MOAT) && Board::isMoatIndex(*it) ? 100 : 0);
 
 		// new cell
 		if(0 > list[*it].prnt)
@@ -293,6 +279,14 @@ Battle::Indexes Battle::Board::GetAStarPath(const Unit & b, const Position & dst
 	if(result.size() > b.GetSpeed())
 	    result.resize(b.GetSpeed());
 
+	// skip moat position
+	if(castle && castle->isBuild(BUILD_MOAT) && ! Board::isMoatIndex(b.GetHeadIndex()))
+	{
+	    Indexes::iterator moat = std::find_if(result.begin(), result.end(), Board::isMoatIndex);
+	    if(moat != result.end())
+		result.resize(std::distance(result.begin(), ++moat));
+	}
+
 	// set passable info
 	for(Indexes::iterator
 	    it = result.begin(); it != result.end(); ++it)
@@ -312,11 +306,11 @@ Battle::Indexes Battle::Board::GetAStarPath(const Unit & b, const Position & dst
 	    }
 	}
     }
-    else
-    if(debug)
+
+    if(debug && result.empty())
     {
 	DEBUG(DBG_BATTLE, DBG_WARN, "path not found: " << b.String() << ", dst: " <<
-	    ", (head: " << dst.GetHead()->GetIndex() << ", tail: " << (b.isWide() ? dst.GetTail()->GetIndex() : -1) << ")");
+	    "(head: " << dst.GetHead()->GetIndex() << ", tail: " << (dst.GetTail() ? dst.GetTail()->GetIndex() : -1) << ")");
     }
 
     return result;
