@@ -23,36 +23,12 @@
 #ifndef H2GAMEIO_H
 #define H2GAMEIO_H
 
-#include "sdlnet.h"
 #include "game.h"
 
-namespace Maps {class Tiles; class FileInfo; struct Addons; }
-class Kingdom;
-class Castle;
-class Heroes;
-class HeroBase;
-class Players;
+namespace Maps { class FileInfo; }
 
 namespace Game
 {
-    class IOld : public QueueMessage
-    {
-    public:
-	IOld(){}
-
-	bool LoadSAV(const std::string &);
-
-	static bool LoadBIN(QueueMessage &);
-
-	static void UnpackTile(QueueMessage &, Maps::Tiles &, u32, u16 = -1);
-	static void UnpackTileAddons(QueueMessage &, Maps::Addons &, u16 = -1);
-	static void UnpackKingdom(QueueMessage &, Kingdom &, u16 = -1);
-	static void UnpackCastle(QueueMessage &, Castle &, u16 = -1);
-	static void UnpackHeroes(QueueMessage &, Heroes &, u16 = -1);
-	static void UnpackHeroBase(QueueMessage &, HeroBase &, u16 = -1);
-	static void UnpackPlayers(QueueMessage &, Players &, u16 = -1);
-    };
-
     bool Save(const std::string &);
     bool Load(const std::string &);
     bool LoadSAV2FileInfo(const std::string &,  Maps::FileInfo &);
