@@ -210,7 +210,7 @@ void Interface::GameArea::Redraw(Surface & dst, u8 flag, const Rect & rt) const
 	for(; it1 != it2; ++it1)
 	{
 	    from = (*it1).GetIndex();
-    	    const Point mp(from % world.w(), from / world.h());
+    	    const Point mp = Maps::GetPoint(from);
 
  	    ++it3;
 	    --green;
@@ -220,7 +220,7 @@ void Interface::GameArea::Redraw(Surface & dst, u8 flag, const Rect & rt) const
 
 	    const u16 index = (it3 == it2 ? 0 :
 		    Route::Path::GetIndexSprite((*it1).direction, (*it3).direction, 
-			Maps::Ground::GetBasePenalty(from, hero->GetLevelSkill(Skill::Secondary::PATHFINDING))));
+			Maps::Ground::GetPenalty(from, Direction::CENTER, hero->GetLevelSkill(Skill::Secondary::PATHFINDING))));
 
 	    const Sprite & sprite = AGG::GetICN(0 > green ? ICN::ROUTERED : ICN::ROUTE, index);
 	    BlitOnTile(dst, sprite, sprite.x() - 14, sprite.y(), mp);
@@ -444,7 +444,7 @@ void Interface::GameArea::GenerateUltimateArtifactAreaSurface(const s32 index, S
 
 	const Rect & rectMaps = gamearea.GetRectMaps();
 	const Rect & areaPosition = gamearea.GetArea();
-	Point pt(index % world.w(), index / world.h());
+	Point pt = Maps::GetPoint(index);
 
         gamearea.SetCenter(pt);
 	gamearea.Redraw(sf, LEVEL_BOTTOM | LEVEL_TOP);
