@@ -226,7 +226,7 @@ Dialog::answer_t Heroes::OpenDialog(bool readonly, bool fade)
     dst_pt.x = cur_pt.x + 49;
     dst_pt.y = cur_pt.y + 130;
 
-    AGG::GetICN(ICN::CREST, Color::NONE == color ? Color::GetIndex(Settings::Get().CurrentColor()) : Color::GetIndex(color)).Blit(dst_pt);
+    AGG::GetICN(ICN::CREST, Color::NONE == GetColor() ? Color::GetIndex(Settings::Get().CurrentColor()) : Color::GetIndex(GetColor())).Blit(dst_pt);
 
     // monster
     dst_pt.x = cur_pt.x + 156;
@@ -299,7 +299,7 @@ Dialog::answer_t Heroes::OpenDialog(bool readonly, bool fade)
 	buttonDismiss.SetDisable(true);
     }
 
-    if(readonly || 2 > world.GetKingdom(GetColor()).GetHeroes().size())
+    if(readonly || 2 > GetKingdom().GetHeroes().size())
     {
         buttonNextHero.Press();
         buttonPrevHero.Press();
