@@ -23,6 +23,7 @@
 #include <cstdlib>
 #include <map>
 #include "maps.h"
+#include "ai.h"
 #include "world.h"
 #include "direction.h"
 #include "settings.h"
@@ -31,22 +32,25 @@
 
 struct cell_t
 {
-    cell_t() : cost_g(MAXU16), cost_t(MAXU16), cost_d(MAXU16), passbl(0), parent(-1), open(true){};
+    cell_t() : cost_g(MAXU16), cost_t(MAXU16), cost_d(MAXU16), passbl(0), open(1), direct(Direction::CENTER), parent(-1) {}
 
-    u16		cost_g;
-    u16		cost_t;
-    u16		cost_d;
+    u16		cost_g;	// ground
+    u16		cost_t; // total
+    u16		cost_d; // distance
     u16		passbl;
+    u16		open;	// bool
+    u16		direct;
     s32		parent;
-    bool	open;
 };
 
-
-u32 GetCurrentLength(std::map<s32, cell_t> & list, s32 cur)
+u16 GetCurrentLength(std::map<s32, cell_t> & list, s32 from)
 {
-    u32 res = 0;
-    const cell_t* cell = &list[cur];
-    while(-1 != cell->parent){ cell = &list[cell->parent]; ++res; };
+    u16 res = 0;
+    while(0 <= list[from].parent)
+    {
+	from = list[from].parent;
+	++res;
+    }
     return res;
 }
 
@@ -56,31 +60,31 @@ bool CheckMonsterProtectionAndNotDst(const s32 & to, const s32 & dst)
     return monsters.size() && monsters.end() == std::find(monsters.begin(), monsters.end(), dst);
 }
 
-bool PassableToTile(const Heroes* hero, const Maps::Tiles & toTile, const Direction::vector_t & direct, const s32 & dst)
+bool PassableToTile(const Heroes & hero, const Maps::Tiles & toTile, const Direction::vector_t & direct, const s32 & dst)
 {
     // check end point
     if(toTile.GetIndex() == dst)
     {
 	// fix toTilePassable with action object
-	if(hero && MP2::isPickupObject(toTile.GetObject()))
+	if(MP2::isPickupObject(toTile.GetObject()))
 	    return true;
 
 	// check direct to object
-	if(MP2::isActionObject(toTile.GetObject(false), (hero ? hero->isShipMaster() : false)))
+	if(MP2::isActionObject(toTile.GetObject(false), hero.isShipMaster()))
 	    return Direction::Reflect(direct) & toTile.GetPassable();
 
 	if(MP2::OBJ_HEROES == toTile.GetObject())
-	    return toTile.isPassable(NULL, Direction::Reflect(direct), false);
+	    return toTile.isPassable(NULL, Direction::Reflect(direct), (CONTROL_AI == hero.GetControl() ? AI::HeroesSkipFog() : false));
     }
 
     // check to tile direct
-    if(! toTile.isPassable(hero, Direction::Reflect(direct), false))
+    if(! toTile.isPassable(&hero, Direction::Reflect(direct), (CONTROL_AI == hero.GetControl() ? AI::HeroesSkipFog() : false)))
 	return false;
 
     if(toTile.GetIndex() != dst)
     {
 	if(MP2::isPickupObject(toTile.GetObject()) ||
-	    MP2::isActionObject(toTile.GetObject(false), (hero ? hero->isShipMaster() : false)))
+	    MP2::isActionObject(toTile.GetObject(false), hero.isShipMaster()))
 	    return false;
 
 	// check hero/monster on route
@@ -101,15 +105,15 @@ bool PassableToTile(const Heroes* hero, const Maps::Tiles & toTile, const Direct
     return true;
 }
 
-bool PassableFromToTile(const Heroes* hero, const s32 & from, const s32 & to, const Direction::vector_t & direct, const s32 & dst)
+bool PassableFromToTile(const Heroes & hero, const s32 & from, const s32 & to, const Direction::vector_t & direct, const s32 & dst)
 {
     const Maps::Tiles & fromTile = world.GetTiles(from);
     const Maps::Tiles & toTile = world.GetTiles(to);
 
     // check start point
-    if(hero && hero->GetIndex() == from)
+    if(hero.GetIndex() == from)
     {
-	if(MP2::isActionObject(fromTile.GetObject(false), hero->isShipMaster()))
+	if(MP2::isActionObject(fromTile.GetObject(false), hero.isShipMaster()))
 	{
 	    // check direct from object
 	    if(! (direct & fromTile.GetPassable()))
@@ -118,13 +122,13 @@ bool PassableFromToTile(const Heroes* hero, const s32 & from, const s32 & to, co
 	else
 	{
 	    // check from tile direct
-	    if(! fromTile.isPassable(hero, direct, false))
+	    if(! fromTile.isPassable(&hero, direct, (CONTROL_AI == hero.GetControl() ? AI::HeroesSkipFog() : false)))
 		return false;
 	}
     }
     else
     {
-	if(MP2::isActionObject(fromTile.GetObject(), (hero ? hero->isShipMaster() : false)))
+	if(MP2::isActionObject(fromTile.GetObject(), hero.isShipMaster()))
 	{
 	    // check direct from object
 	    if(! (direct & fromTile.GetPassable()))
@@ -133,7 +137,7 @@ bool PassableFromToTile(const Heroes* hero, const s32 & from, const s32 & to, co
 	else
 	{
 	    // check from tile direct
-	    if(! fromTile.isPassable(hero, direct, false))
+	    if(! fromTile.isPassable(&hero, direct, (CONTROL_AI == hero.GetControl() ? AI::HeroesSkipFog() : false)))
 		return false;
 	}
     }
@@ -171,9 +175,17 @@ bool PassableFromToTile(const Heroes* hero, const s32 & from, const s32 & to, co
     return PassableToTile(hero, toTile, direct, dst);
 }
 
-bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 to, const u16 limit, const Heroes* hero)
+u16 GetPenaltyFromTo(const s32 & from, const s32 & to, const Direction::vector_t & direct, const u8 & pathfinding)
 {
-    const u8 pathfinding = (hero ? hero->GetLevelSkill(Skill::Secondary::PATHFINDING) : Skill::Level::NONE);
+    const u16 & cost1 = Maps::Ground::GetPenalty(from, direct, pathfinding); // penalty: for [cur] out
+    const u16 & cost2 = Maps::Ground::GetPenalty(to, Direction::Reflect(direct), pathfinding); // penalty: for [tmp] in
+    return (cost1 + cost2) >> 1;
+}
+
+bool Route::Path::Find(const s32 & to, const u16 limit)
+{
+    const u8 pathfinding = hero.GetLevelSkill(Skill::Secondary::PATHFINDING);
+    const s32 & from = hero.GetIndex();
 
     s32 cur = from;
     s32 alt = 0;
@@ -181,18 +193,20 @@ bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 t
     std::map<s32, cell_t> list;
     std::map<s32, cell_t>::iterator it1 = list.begin();
     std::map<s32, cell_t>::iterator it2 = list.end();
-    Direction::vector_t direct = Direction::CENTER;
 
     list[cur].cost_g = 0;
     list[cur].cost_t = 0;
     list[cur].parent = -1;
-    list[cur].open   = false;
+    list[cur].open   = 0;
+
+    clear();
 
     while(cur != to)
     {
 	LocalEvent::Get().HandleEvents(false);
 
-	for(direct = Direction::TOP_LEFT; direct != Direction::CENTER; ++direct)
+	for(Direction::vector_t
+	    direct = Direction::TOP_LEFT; direct != Direction::CENTER; ++direct)
 	{
     	    if(Maps::isValidDirection(cur, direct))
 	    {
@@ -200,45 +214,43 @@ bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 t
 
 		if(list[tmp].open)
 		{
+		    const u16 costg = GetPenaltyFromTo(cur, tmp, direct, pathfinding);
+
 		    // new
 		    if(-1 == list[tmp].parent)
 		    {
-	    		const u16 costg = Maps::Ground::GetPenalty(tmp, direct, pathfinding);
-			if(MAXU16 == costg) continue;
-
 			if((list[cur].passbl & direct) ||
 			   PassableFromToTile(hero, cur, tmp, direct, to))
 			{
 			    list[cur].passbl |= direct;
 
-			    cell_t & cell = list[tmp];
-	    		    cell.cost_g = costg;
-			    cell.parent = cur;
-			    cell.open = true;
-	    		    cell.cost_t = cell.cost_g + list[cur].cost_t;
-			    cell.cost_d = 50 * Maps::GetApproximateDistance(tmp, to);
+			    list[tmp].direct = direct;
+	    		    list[tmp].cost_g = costg;
+			    list[tmp].parent = cur;
+			    list[tmp].open   = 1;
+			    list[tmp].cost_d = 50 * Maps::GetApproximateDistance(tmp, to);
+	    		    list[tmp].cost_t = list[cur].cost_t + costg;
 			}
 		    }
 		    // check alt
 		    else
 		    {
-			alt = Maps::Ground::GetPenalty(cur, direct, pathfinding);
-			if(list[tmp].cost_t > list[cur].cost_t + alt &&
-			   ((list[cur].passbl & direct) ||
-			     PassableFromToTile(hero, cur, tmp, direct, to)))
+			if(list[tmp].cost_t > list[cur].cost_t + costg &&
+			   ((list[cur].passbl & direct) || PassableFromToTile(hero, cur, tmp, direct, to)))
 			{
 			    list[cur].passbl |= direct;
 
+			    list[tmp].direct = direct;
 			    list[tmp].parent = cur;
-			    list[tmp].cost_g = alt;
-			    list[tmp].cost_t = list[cur].cost_t + alt;
+			    list[tmp].cost_g = costg;
+			    list[tmp].cost_t = list[cur].cost_t + costg;
 			}
 		    }
     		}
 	    }
 	}
 
-	list[cur].open = false;
+	list[cur].open = 0;
 
 	it1 = list.begin();
 	alt = -1;
@@ -253,7 +265,8 @@ bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 t
 #ifdef WITH_DEBUG
 	    if(IS_DEBUG(DBG_OTHER, DBG_TRACE) && cell2.cost_g != MAXU16)
 	    {
-		direct = Direction::Get(cur, (*it1).first);
+		const u16 direct = Direction::Get(cur, (*it1).first);
+
 		if(Direction::UNKNOWN != direct)
 		{
 		    VERBOSE("\t\tdirect: " << Direction::String(direct) <<
@@ -273,12 +286,14 @@ bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 t
 	}
 
 	// not found, and exception
-	if(MAXU16 == tmp || -1 == alt || (limit && GetCurrentLength(list, cur) > limit)) break;
+	if(MAXU16 == tmp || -1 == alt) break;
 #ifdef WITH_DEBUG
 	else
 	DEBUG(DBG_OTHER, DBG_TRACE, "select: " << alt);
 #endif
 	cur = alt;
+
+	if(GetCurrentLength(list, cur) > limit) break;
     }
 
     // save path
@@ -286,16 +301,14 @@ bool Route::PathFind(std::list<Route::Step>* result, const s32 from, const s32 t
     {
 	while(cur != from)
 	{
-	    if(-1 == list[cur].parent) break;
-	    alt = cur;
-    	    cur = list[alt].parent;
-	    if(result) result->push_front(Route::Step(cur, Direction::Get(cur, alt), list[alt].cost_g));
+	    push_front(Route::Step(list[cur].parent, list[cur].direct, list[cur].cost_g));
+    	    cur = list[cur].parent;
 	}
-        return true;
+    }
+    else
+    {
+	DEBUG(DBG_OTHER, DBG_TRACE, "not found" << ", from:" << from << ", to: " << to);
     }
 
-    DEBUG(DBG_OTHER, DBG_TRACE, "not found" << ", from:" << from << ", to: " << to);
-    list.clear();
-
-    return false;
+    return !empty();
 }
