@@ -749,10 +749,7 @@ void World::LoadMaps(const std::string &filename)
 				hero = vec_heroes.GetFreeman(race);
 
 			    if(hero)
-			    {
 				hero->LoadFromMP2(findobject, pblock, color, race);
-				hero->SetModes(Heroes::HUNTER);
-			    }
 			}
 			else
 			{
@@ -1158,7 +1155,7 @@ void World::MonthOfMonstersAction(const Monster & mons)
 	    for(MapsIndexes::const_iterator
 		it = objv.begin(); it != objv.end(); ++it)
 	    {
-		const MapsIndexes & obja = Maps::GetDistanceIndexes(*it, dist);
+		const MapsIndexes & obja = Maps::GetAroundIndexes(*it, dist);
 		excld.insert(excld.end(), obja.begin(), obja.end());
 	    }
 	}
@@ -1175,7 +1172,7 @@ void World::MonthOfMonstersAction(const Monster & mons)
 		excld.end() == std::find(excld.begin(), excld.end(), tile.GetIndex()))
 	    {
 		tiles.push_back(tile.GetIndex());
-		const MapsIndexes & obja = Maps::GetDistanceIndexes(tile.GetIndex(), dist);
+		const MapsIndexes & obja = Maps::GetAroundIndexes(tile.GetIndex(), dist);
 		excld.insert(excld.end(), obja.begin(), obja.end());
 	    }
 	}
