@@ -359,6 +359,42 @@ bool Maps::TilesAddon::isRoad(u16 direct) const
 {
     switch(MP2::GetICNObject(object))
     {
+	// from sprite road
+	case ICN::ROAD:
+	    if(0 == index ||
+		4 == index ||
+		5 == index ||
+		13 == index ||
+		26 == index) return direct & (Direction::TOP | Direction::BOTTOM);
+	    else
+	    if(2 == index ||
+		21 == index ||
+		28 == index) return direct & (Direction::LEFT | Direction::RIGHT);
+	    else
+	    if(17 == index ||
+		29 == index) return direct & (Direction::TOP_LEFT | Direction::BOTTOM_RIGHT);
+	    else
+	    if(18 == index ||
+		30 == index) return direct & (Direction::TOP_RIGHT | Direction::BOTTOM_LEFT);
+	    else
+	    if(3 == index) return direct & (Direction::TOP | Direction::BOTTOM | Direction::LEFT | Direction::RIGHT);
+	    else
+	    if(6 == index) return direct & (Direction::TOP | Direction::BOTTOM | Direction::RIGHT);
+	    else
+	    if(7 == index) return direct & (Direction::TOP | Direction::RIGHT);
+	    else
+	    if(9 == index) return direct & (Direction::BOTTOM | Direction::RIGHT);
+	    else
+	    if(12 == index) return direct & (Direction::BOTTOM | Direction::LEFT);
+	    else
+	    if(14 == index) return direct & (Direction::TOP | Direction::BOTTOM | Direction::LEFT);
+	    else
+	    if(16 == index) return direct & (Direction::TOP | Direction::LEFT);
+	    else
+	    if(19 == index) return direct & (Direction::TOP_LEFT | Direction::BOTTOM_RIGHT);
+	    else
+	    if(20 == index) return direct & (Direction::TOP_RIGHT | Direction::BOTTOM_LEFT);
+
 	// castle and tower (gate)
 	case ICN::OBJNTOWN:
 	    if(13 == index ||
@@ -385,34 +421,6 @@ bool Maps::TilesAddon::isRoad(u16 direct) const
 		67 == index ||
 		77 == index) return direct & (Direction::TOP | Direction::BOTTOM);
 
-	// from sprite road
-	case ICN::ROAD:
-	    if(0  == index ||
-	       4  == index ||
-	       5  == index ||
-	       7  == index ||
-	       9  == index ||
-	       12 == index ||
-	       13 == index ||
-	       16 == index ||
-	       19 == index ||
-	       20 == index ||
-	       26 == index) return direct & (Direction::TOP | Direction::BOTTOM);
-	    else
-	    if(2  == index ||
-	       21 == index ||
-	       28 == index) return direct & (Direction::RIGHT | Direction::LEFT);
-	    else
-	    if(3  == index ||
-	       6  == index ||
-	       14 == index) return direct & (Direction::TOP | Direction::BOTTOM | Direction::RIGHT | Direction::LEFT);
-	    else
-	    if(17 == index ||
-	       29 == index) return direct & (Direction::TOP_LEFT | Direction::BOTTOM_RIGHT);
-	    else
-	    if(18 == index ||
-	       30 == index) return direct & (Direction::TOP_RIGHT | Direction::BOTTOM_LEFT);
-	
 	default:
 	    break;
     }
@@ -1394,7 +1402,7 @@ void Maps::Tiles::Remove(u32 uniq)
 void Maps::Tiles::RedrawTile(Surface & dst) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if(area.GetRectMaps() & mp)
 	area.BlitOnTile(dst, GetTileSurface(), 0, 0, mp);
@@ -1403,7 +1411,7 @@ void Maps::Tiles::RedrawTile(Surface & dst) const
 void Maps::Tiles::RedrawBottom(Surface & dst, bool skip_objs) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if((area.GetRectMaps() & mp) &&
 	!addons_level1.empty())
@@ -1440,7 +1448,7 @@ void Maps::Tiles::RedrawPassable(Surface & dst) const
 {
 #ifdef WITH_DEBUG
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if(area.GetRectMaps() & mp)
     {
@@ -1477,7 +1485,7 @@ void Maps::Tiles::RedrawObjects(Surface & dst) const
 void Maps::Tiles::RedrawMonster(Surface & dst) const
 {
     const Settings & conf = Settings::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
     const Interface::GameArea & area = Interface::GameArea::Get();
     s32 dst_index = -1;
 
@@ -1535,7 +1543,7 @@ void Maps::Tiles::RedrawMonster(Surface & dst) const
 
 void Maps::Tiles::RedrawBoat(Surface & dst) const
 {
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
     const Interface::GameArea & area = Interface::GameArea::Get();
 
     if(!(area.GetRectMaps() & mp)) return;
@@ -1591,7 +1599,7 @@ bool SkipRedrawTileBottom4Hero(const Maps::TilesAddon & ta, const u16 & passable
 void Maps::Tiles::RedrawBottom4Hero(Surface & dst) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if((area.GetRectMaps() & mp) &&
 	!addons_level1.empty())
@@ -1622,7 +1630,7 @@ void Maps::Tiles::RedrawBottom4Hero(Surface & dst) const
 void Maps::Tiles::RedrawTop(Surface & dst, const TilesAddon* skip) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if(!(area.GetRectMaps() & mp)) return;
 
@@ -1679,7 +1687,7 @@ void Maps::Tiles::RedrawTop(Surface & dst, const TilesAddon* skip) const
 void Maps::Tiles::RedrawTop4Hero(Surface & dst, bool skip_ground) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     if(!(area.GetRectMaps() & mp)) return;
 
@@ -1751,7 +1759,17 @@ std::string Maps::Tiles::String(void) const
 	"----------------:--------" << std::endl <<
 	"maps index      : " << GetIndex() << std::endl <<
 	"tile index      : " << TileSpriteIndex() << std::endl <<
-	"ground          : " << Ground::String(GetGround()) << (isRoad() ? ", (road)" : "") << std::endl <<
+	"ground          : " << Ground::String(GetGround());
+    if(isRoad())
+    {
+	Addons::const_iterator it = std::find_if(addons_level1.begin(), addons_level1.end(),
+				std::bind2nd(std::mem_fun_ref(&TilesAddon::isRoad), DIRECTION_ALL));
+	os << ", (" << "road";
+	if(ICN::ROAD == MP2::GetICNObject((*it).object))
+	    os << ", " << "index: " << static_cast<int>((*it).index);
+	os << ")";
+    }
+    os << std::endl <<
 	"passable        : " << (tile_passable ? Direction::String(tile_passable) : "false");
 #ifdef WITH_DEBUG
 	if(passable_disable)
@@ -1877,31 +1895,29 @@ bool TileIsGround(s32 index, u16 ground)
 }
 
 /* accept move */
-bool Maps::Tiles::isPassable(const Heroes* hero, u16 direct, bool skipfog) const
+bool Maps::Tiles::isPassable(const Heroes & hero) const
 {
-    if(!skipfog && isFog(Settings::Get().CurrentColor()))
-	return false;
-
-    if(hero)
+    if(hero.isShipMaster())
     {
-	if(hero->isShipMaster())
-	{
-	    if(! isWater())
-		return false;
+	if(! isWater())
+	    return false;
 
-	    if(MP2::OBJ_BOAT == GetObject())
-		return false;
-	}
-	else
-	// if(! hero->isShipMaster() &&
-	if(isWater())
+	if(MP2::OBJ_BOAT == GetObject())
+	    return false;
+    }
+    else
+    // if(! hero->isShipMaster() &&
+    if(isWater())
+    {
+	switch(GetObject())
 	{
 	    // fix shipwreck: place on water
-	    if(MP2::OBJ_SHIPWRECK == GetObject())
-		return direct & tile_passable;
-	    else
+	    case MP2::OBJ_SHIPWRECK:
+		// check later
+		break;
+
 	    // for: meetings/attack hero
-	    if(MP2::OBJ_HEROES == GetObject())
+	    case MP2::OBJ_HEROES:
 	    {
 		// scan ground
 		const MapsIndexes & v = Maps::GetAroundIndexes(GetIndex());
@@ -1909,11 +1925,25 @@ bool Maps::Tiles::isPassable(const Heroes* hero, u16 direct, bool skipfog) const
 		    std::not1(std::bind2nd(std::ptr_fun(&TileIsGround), Ground::WATER))))
 		    return false;
 	    }
-	    else
+	    break;
+
+	    default:
+		// ! hero->isShipMaster() && isWater()
 		return false;
 	}
     }
 
+    return true;
+}
+
+bool Maps::Tiles::isPassable(const Heroes* hero, u16 direct, bool skipfog) const
+{
+    if(!skipfog && isFog(Settings::Get().CurrentColor()))
+	return false;
+
+    if(hero && ! isPassable(*hero))
+	return false;
+
     return direct & tile_passable;
 }
 
@@ -2499,7 +2529,7 @@ void Maps::Tiles::ClearFog(u8 colors)
 void Maps::Tiles::RedrawFogs(Surface & dst, u8 color) const
 {
     const Interface::GameArea & area = Interface::GameArea::Get();
-    const Point mp(GetIndex() % world.w(), GetIndex() / world.w());
+    const Point mp = Maps::GetPoint(GetIndex());
 
     // get direction around foga
     u16 around = 0;
