@@ -774,7 +774,7 @@ std::string Settings::GetLangDir(void)
     return "";
 }
 
-std::string Settings::GetSaveDir(void)
+std::string Settings::GetWriteableDir(const char* subdir)
 {
     std::string res;
     const ListDirs dirs = GetRootDirs();
@@ -782,7 +782,7 @@ std::string Settings::GetSaveDir(void)
     for(ListDirs::const_iterator
 	it = dirs.begin(); it != dirs.end(); ++it)
     {
-	res = *it + SEPARATOR + "files" + SEPARATOR + "save";
+	res = *it + SEPARATOR + "files" + SEPARATOR + subdir;
         if(IsDirectory(res, true)) return res;
     }
 
@@ -791,6 +791,11 @@ std::string Settings::GetSaveDir(void)
     return "";
 }
 
+std::string Settings::GetSaveDir(void)
+{
+    return GetWriteableDir("save");
+}
+
 /* return path to locales directory */
 const std::string & Settings::PlayMusCommand(void) const { return playmus_command; }
 
