@@ -39,8 +39,9 @@
 #include <android/log.h>
 #endif
 
+#define FORMAT_VERSION_2850 0x0B22
 #define FORMAT_VERSION_2830 0x0B0E
-#define CURRENT_FORMAT_VERSION FORMAT_VERSION_2830
+#define CURRENT_FORMAT_VERSION FORMAT_VERSION_2850
 #define LAST_FORMAT_VERSION FORMAT_VERSION_2830
 
 enum
@@ -427,6 +428,7 @@ public:
     static ListFiles GetListFiles(const std::string & prefix, const std::string & filter);
     static ListDirs GetRootDirs(void);
     static std::string GetLastFile(const std::string & prefix, const std::string & name);
+    static std::string GetWriteableDir(const char*);
     static std::string GetSaveDir(void);
     static std::string GetLangDir(void);
     static std::string GetHomeDir(void);
